/*
Pluck: an open source trivia game for Android

Copyright (C) 2021 Joguitos do luca0N!

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.

This game is a fork of LibreTrivia, and its source code is available at
<https://github.com/tryton-vanmeer/LibreTrivia>.

Contact us at <joguitos+pluck@luca0n.com>.
*/

package com.luca0n.joguitos.pluck.activities;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

import com.mikepenz.aboutlibraries.Libs;
import com.mikepenz.aboutlibraries.LibsBuilder;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.BufferedReader;
import java.nio.charset.StandardCharsets;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.app.AlertDialog;
import com.luca0n.joguitos.pluck.R;
import com.luca0n.joguitos.pluck.settings.SettingsActivity;
import com.luca0n.joguitos.pluck.util.UiUtil;

@SuppressLint("Registered")
public class BaseActivity extends AppCompatActivity {
	public static final int REQUEST_ACTIVITY_CLOSED = 1;

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.app_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.settings:
				onSettings();
				return true;
			case R.id.license:
				onLicense();
				return true;
			case R.id.about:
				onAbout();
				return true;
			case android.R.id.home:
				onBackPressed();
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent extras){
		switch (requestCode){
			case REQUEST_ACTIVITY_CLOSED:
				if (resultCode == Activity.RESULT_OK){
					// Check if the theme changed.
					recreate();
				}
				break;
			default:
				super.onActivityResult(requestCode, resultCode, extras);
				break;
		}
	}

	/**
	 * Shows an AlertDialog with the GNU GPL v3 on it.
	 * @since 2021-03-29
	 */
	private void onLicense() {
		StringBuilder license = new StringBuilder();
		boolean failed = false;
		// Get the license file.
		try {
			InputStream s = getResources().openRawResource(R.raw.license);
			Reader r = new BufferedReader(new InputStreamReader(s, StandardCharsets.UTF_8));
			for(int b; (b = r.read()) >= 0;)
				license.append((char) b);
		} catch (IOException e){
			failed = true;
			license.append(getString(R.string.error_license_io_error));
		}
		AlertDialog.Builder b = new AlertDialog.Builder(this)
			.setTitle(R.string.ui_license)
			.setMessage(license.toString())
			.setPositiveButton(android.R.string.ok, null);
		if (failed){
			b.setIcon(android.R.drawable.ic_dialog_alert);
			b.setTitle(R.string.error);
		}
		b.show();
	}

	private void onSettings() {
		Intent intent = new Intent(this, SettingsActivity.class);
		startActivityForResult(intent, REQUEST_ACTIVITY_CLOSED);
	}

	private void onAbout() {
		String appName = getResources().getString(R.string.app_name);
		String appDescription = getResources().getString(R.string.app_description);

		new LibsBuilder()
			.withActivityStyle(UiUtil.getTheme(this).equals(getString(R.string.pref_ui_theme_entryValues_dark)) ? Libs.ActivityStyle.DARK : Libs.ActivityStyle.LIGHT)
			.withAboutIconShown(true)
			.withAboutAppName(appName)
			.withAboutVersionShownName(true)
			.withAboutDescription(appDescription)
			.start(this);
	}
}
